@@ -95,7 +95,7 @@ let rec unwrap e =
   match e with
   | Wrap (loc, e, an) -> unwrap e
   | Shared (loc, e, an) -> not_implemented loc "cyclic references"
-  | Tvar (loc, name) -> not_implemented loc "parametrized type"
+  | Tvar _
   | Sum _
   | Record _
   | Tuple _
@@ -181,15 +181,19 @@ let double_esc s =
   escape_string_content Double s
 
 let runtime_start atd_filename =
-  sprintf {|// Generated by atdts from type definitions in '%s'.
-//
-// Type-safe translations from/to JSON
-//
-// For each type 'Foo', there is a pair of functions:
-// - 'writeFoo': convert a 'Foo' value into a JSON-compatible value.
-// - 'readFoo': convert a JSON-compatible value into a TypeScript value
-//   of type 'Foo'.
-|}
+  sprintf {|/*
+  Generated by atdts from type definitions in '%s'.
+
+  Type-safe translations from/to JSON
+
+  For each type 'Foo', there is a pair of functions:
+  - 'writeFoo': convert a 'Foo' value into a JSON-compatible value.
+  - 'readFoo': convert a JSON-compatible value into a TypeScript value
+    of type 'Foo'.
+*/
+
+/* tslint:disable */
+/* eslint-disable */|}
     atd_filename
 
 let runtime_end = {|
@@ -197,8 +201,6 @@ let runtime_end = {|
 // Runtime library
 /////////////////////////////////////////////////////////////////////
 
-export type Int = number
-
 export type Option<T> = null | { value: T }
 
 function _atd_missing_json_field(type_name: string, json_field_name: string) {
@@ -231,7 +233,7 @@ function _atd_bad_ts(expected_type: string, ts_value: any, context: any) {
                   ` Occurs in '${JSON.stringify(context)}'.`)
 }
 
-function _atd_check_json_tuple(len: Int, x: any, context: any) {
+function _atd_check_json_tuple(len: number /*int*/, x: any, context: any) {
   if (! Array.isArray(x) || x.length !== len)
     _atd_bad_json('tuple of length ' + len, x, context);
 }
@@ -254,7 +256,7 @@ function _atd_read_bool(x: any, context: any): boolean {
   }
 }
 
-function _atd_read_int(x: any, context: any): Int {
+function _atd_read_int(x: any, context: any): number /*int*/ {
   if (Number.isInteger(x))
     return x
   else {
@@ -435,7 +437,7 @@ function _atd_write_bool(x: any, context: any): boolean {
   }
 }
 
-function _atd_write_int(x: any, context: any): Int {
+function _atd_write_int(x: any, context: any): number /*int*/ {
   if (Number.isInteger(x))
     return x
   else {
@@ -544,7 +546,7 @@ function _atd_write_required_field<T>(type_name: string,
 }
 
 function _atd_write_optional_field<T>(write_elt: (x: T, context: any) => any,
-                                      x: T,
+                                      x: T | undefined,
                                       context: any): any {
   if (x === undefined || x === null)
     return x
@@ -596,7 +598,7 @@ let ts_type_name env (name : string) =
   match name with
   | "unit" -> "Null"
   | "bool" -> "boolean"
-  | "int" -> "Int"
+  | "int" -> "number /*int*/"
   | "float" -> "number"
   | "string" -> "string"
   | "abstract" -> "any"
@@ -625,7 +627,7 @@ let rec type_name_of_expr env (e : type_expr) : string =
   | Shared (loc, e, an) -> not_implemented loc "shared"
   | Wrap (loc, e, an) -> todo "wrap"
   | Name (loc, (loc2, name, []), an) -> ts_type_name env name
-  | Name (loc, _, _) -> not_implemented loc "parametrized types"
+  | Name (loc, _, _) -> assert false
   | Tvar (loc, _) -> not_implemented loc "type variables"
 
 and type_name_of_tuple env cells : string =
@@ -708,9 +710,9 @@ let rec json_reader env e =
   | Name (loc, (loc2, name, []), an) ->
       (match name with
        | "bool" | "int" | "float" | "string" -> sprintf "_atd_read_%s" name
-       | "abstract" -> "((x: any): any => x)"
+       | "abstract" -> "((x: any, context): any => x)"
        | _ -> reader_name env name)
-  | Name (loc, _, _) -> not_implemented loc "parametrized types"
+  | Name (loc, _, _) -> assert false
   | Tvar (loc, _) -> not_implemented loc "type variables"
 
 (*
@@ -756,7 +758,7 @@ let rec json_writer env e =
   | Name (loc, (loc2, name, []), an) ->
       (match name with
        | "bool" | "int" | "float" | "string" -> sprintf "_atd_write_%s" name
-       | "abstract" -> "((x: any): any => x)"
+       | "abstract" -> "((x: any, context): any => x)"
        | _ -> writer_name env name)
   | Name (loc, _, _) -> not_implemented loc "parametrized types"
   | Tvar (loc, _) -> not_implemented loc "type variables"
@@ -1187,9 +1189,15 @@ let run_file src_path =
     |> String.lowercase_ascii
   in
   let dst_path = dst_name in
-  let (atd_head, atd_module), _original_types =
+  let full_module, _original_types =
     Atd.Util.load_file
       ~annot_schema
-      ~expand:false ~inherit_fields:true ~inherit_variants:true src_path
+      ~expand:true (* monomorphization *)
+      ~keep_builtins:true
+      ~inherit_fields:true
+      ~inherit_variants:true
+      src_path
   in
+  let full_module = Atd.Ast.use_only_specific_variants full_module in
+  let atd_head, atd_module = full_module in
   to_file ~atd_filename:src_name atd_module dst_path
